
package sub_arctic.input;

import sub_arctic.lib.interactor;
import sub_arctic.lib.sub_arctic_error;

/** 
 * This class implements a focus based agent that tracks (dispatches, but 
 * never consumes) mouse button press and release events.  This is used by
 * things that need to hear about presses or releases that they wouldn't 
 * normally directly receive (i.e. to reset something).<p>
 *
 * This agent dispatches under the click_tracking input protocol.<p>
 *
 * @author Scott Hudson
 */
public class click_track_agent extends focus_dispatch_agent {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Default constructor */
  public click_track_agent()
    {
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Allow objects in this focus set only if they implement the 
   * click_tracking interface. <p>
   * 
   * @param focusable candidate_obj object we are testing for eligibility
   * @return boolean indicating if object may be added to focus
   */
  public boolean allowable_focus(focusable candidate_obj)
    {
      return candidate_obj instanceof click_tracking;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Only interested in mouse button moves. 
   * @param event evt the event we are expressing interest or non-interest in.
   * @return boolean indicating whether we want the event.
   */
  public boolean event_is_useful(event evt)
    {
      return  evt.id() == event.MOUSE_DOWN || evt.id() == event.MOUSE_UP; 
      // note: the semantics of AWT focus are really not documented, so 
      //       we may also need to be sending for focus events:
      // evt.id() == event.LOST_FOCUS || evt.id() == event.GOT_FOCUS;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Dispatch an event passed to us by our policy.
   *
   * @param event      evt       event that we are dispatching.
   * @param Object     user_info uninterpreted information to pass with event.
   * @param interactor to_obj    ignored.
   * @param int        seq_num   ignored.
   * @return boolean indicating whether event was dispatched and consumed.
   */
  public boolean dispatch_event(
    event            evt,
    Object           user_info,
    interactor       to_obj,
    int              seq_num)
    {
      int i;
      click_tracking dispatch_to_obj;
      boolean result = false;

      /* do a dispatch of raw event to each element of the focus set */
      for (i = 0; i < focus_set_size(); i++)
	{
	  /* extract the object and dispatch to it */
	  dispatch_to_obj = (click_tracking)focus_item(i);
	  if (dispatch_to_obj instanceof interactor)
	    evt.global_to_local((interactor)dispatch_to_obj);
	  else
	    evt.reset_to_global();
	  result |= dispatch_to_obj.track_click(evt, user_info);
	}
      return result;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
