package sub_arctic.input;

import sub_arctic.lib.interactor;
import sub_arctic.lib.interactor_consts;
import sub_arctic.lib.sub_arctic_error;

import java.awt.Point;

/** 
 * Focus agent that implements grow-drag.  Grow_drag is a drag designed for
 * changing the size objects.  The grow_drag protocol provides for 
 * drag_start(), drag_feedback(), and drag_end() methods.  The parameters 
 * passed to these supply an original size and a new size (based on movement
 * horizontally and vertically away from the drag start point).
 *
 * @see sub_arctic.input.grow_draggable
 * @author  Scott Hudson
 */
public class grow_drag_focus_agent extends single_focus_agent {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Default constructor */
  public grow_drag_focus_agent()
    {
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Method to limit the objects allowable as members of the focus set.
   * In this case we only allow things which implement grow_draggable.
   * @param candidate_obj the object we are deciding upon
   * @return a boolean indicating if the object is acceptable
   */
  public boolean allowable_focus(focusable candidate_obj)
    {
      /* must be a grow_draggable */
      return candidate_obj instanceof grow_draggable;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Method to determine if this agent needs to see a given event.  In this
   * case we are interested in MOUSE_DRAG, MOUSE_MOVE and MOUSE_UP only.
   * @param evt the event in question
   * @boolean indication of whether the event should be passed to this agent
   */
  public boolean event_is_useful(event evt)
    {
      /* interested in drag, move, and button up */
      return evt.id() == event.MOUSE_DRAG || evt.id() == event.MOUSE_MOVE || 
	     evt.id() == event.MOUSE_UP;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Start width of the current drag */
  protected int start_w = 0;

  /** Start height of the current drag */
  protected int start_h = 0;

  /** Start x position of the current drag (in global coordinates) */
  protected int start_x = 0;

  /** Start y position of the current drag (in global coordinates) */
  protected int start_y = 0;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Override the normal inform_focus_enter to deliver the drag_start().
   *
   * @param obj       the object we are dragging
   * @param evt       the event that "caused" the start of drag (typically 
   *                  a mouse button down event)
   * @param user_info an uninterpreted object to be passed to the object
   */
  protected void inform_focus_enter(
    focusable obj, 
    event     evt, 
    Object    user_info)
    {
      grow_draggable drag_obj;
      Point          glob_pt, pt;
      event          cur_event;

      /* do a drag_start rather than the default focus_set_enter() */

      /* retype obj to what it is supposed to be */
      drag_obj = (grow_draggable)obj;

      /* make a copy of the event */
      cur_event = new event(evt);

      /* put the event into local coords of the object */
      cur_event.global_to_local(drag_obj);

      /* save initial size and drag position in global coordinates */
      start_w = drag_obj.w();
      start_h = drag_obj.h();
      start_x = evt.global_x();
      start_y = evt.global_y();

      /* dispatch the start of the drag to the object */
      drag_obj.drag_start(cur_event, user_info);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Override the normal inform_focus_exit to deliver the drag_end().
   *
   * @param obj       the object we are dragging
   * @param evt       the event that "caused" the end of drag (typically
   *                  a mouse button up event)
   * @param user_info an uninterpreted object to be passed to the object
   */
  protected void inform_focus_exit(
    focusable obj, 
    event     evt, 
    Object    user_info)
    {
      grow_draggable drag_obj;
      event          cur_event;
      Point          pt, start_pt;
      int            dw, dh;

      /* get object into proper type */
      drag_obj = (grow_draggable)obj;

      /* end the drag */

      /* make a local copy of the current event */
      cur_event = new event(evt);

      /* put event into coords of object we are dispatching to */
      cur_event.global_to_local(drag_obj);

      /* compute delta from start */
      dw = cur_event.global_x() - start_x; 
      dh = cur_event.global_y() - start_y; 

      /* dispatch drag_end to the object */
      drag_obj.drag_end(cur_event, start_w+dw, start_h+dh, start_w, start_h,
			                                             user_info);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Dispatch an event under this protocol.  This will send either a 
   * drag_feedback() or drag_end() to the object.
   *
   * @param evt       the event being dispatched (already approved by 
   *                  event_is_useful())
   * @param user_info an uninterpreted info object to be passed to the 
   *                  dragged object
   * @param to_obj    for focus agents this is always null
   * @param seq_num   for focus agents this is ignored
   * @return value indicating if the event was consumed 
   */
  public boolean dispatch_event(
    event      evt,
    Object     user_info,
    interactor to_obj,
    int        seq_num) 
{
      grow_draggable drag_obj;
      Object         drag_info;
      event          cur_event;
      int            prev_off_x, prev_off_y, off_x, off_y;
      int            dw, dh;
      boolean        result;
      Point          pt, start_pt;

      /* extract our focus item and if there is none, bail out now */
      drag_obj = (grow_draggable)_focus_set.elementAt(0);
      if (drag_obj == null) return false;
      drag_info = _user_info_set.elementAt(0);

      /* If this is end, clear the focus and let inform_focus_exit do work */
      if (evt.id() == event.MOUSE_UP)
	{
	  clear_focus(evt);
	  return true;
	}

      /* make a local copy of the current event */
      cur_event = new event(evt);

      /* put event into coords of object we are dispatching to */
      cur_event.global_to_local(drag_obj);

      /* compute delta from start */
      dw = cur_event.global_x() - start_x;
      dh = cur_event.global_y() - start_y;

      /* dispatch drag_feedback to the object */
      return drag_obj.drag_feedback(cur_event, start_w+dw, start_h+dh, 
					start_w, start_h, drag_info);
    }

   //had:
   //* @exception general

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
