package sub_arctic.input;

import sub_arctic.lib.*;

/**
 * This agent handles the input during modal (pop-up or pull-down) 
 * menus. It is responsible for the sequencing so that submenus
 * are popped down if a parent menu gets a selection. To use this
 * agent, but sure that you use "add_to_focus" and that you add
 * things in the logical order they are "popped up" on the display,
 * i.e. submenus get added after parent menus. <p>
 * 
 * This agent expects that you are going to pop up/down based
 * when you get added to the focus set and when you get removed.
 * Thus, when it thinks you should go away it just removes you
 * from the focus set.<p>
 *
 * @author Ian Smith
 */
public class menu_focus_agent extends focus_dispatch_agent {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This holds the most recently active menu. This allows 
   * us to send an "exit" command to him.
   */
  protected menu_focusable most_recent;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This is where we keep track of what object to notify when
   * the interaction ends.
   */
  protected menu_notifier _notify;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * See if we even fool with this type of event. We only handle
   * drag-related interactions.
   *
   * @param event evt the event to test
   * @return boolean true if we want the event
   */
  public boolean event_is_useful(event evt) {
    /* interested in drag, move, and button up */
    return evt.id() == event.MOUSE_DRAG || evt.id() == event.MOUSE_MOVE || 
      evt.id() == event.MOUSE_UP;
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Send an event to the user's interactors. 
   * @param event      evt       the event to dispatch
   * @param interactor to_obj    the object to whom the event should go
   * @param Object     user_info the object the user provided at the time he 
   *                             entered the focus set
   * @param int        seq_num   the sequence number of this event
   * @return boolean we return true if we dispatched the event
   */
  public  boolean dispatch_event(event evt, Object user_info,
				 interactor to_obj, int seq_num) 
{
     pick_collector p;
     menu_focusable mf;
     event          copy_evt;

     /*
      * Ok, we don't even want to do any processing unless our focus
      * list is non-empty. 
      */
     if (_focus_set.size()==0) {
       return false;
     }

     /* make a copy of the event */
     copy_evt = new event(evt);

     /* ok, we need to figure out who (if anybody) this mouse
	move or release is over ...we use the positional code
	for this! */
     /* find a focused on event */
     mf=find_focused_item(copy_evt);

     /* this looks at the most_recent to see who send exit to, if anyone...
      note that we might send an exit event anytime the most recent object
      and the mf just returned are different */
     generate_exit_event(copy_evt,mf,user_info);

     /* if nobody is being moused on ... */
     if (mf==null) {

       /* it might be a mouse_up.... in which case just drop everyone 
          from the focus list*/
       if (copy_evt.id()==copy_evt.MOUSE_UP) {
	 /* get rid of everybody */
	 clear_focus(copy_evt);
	 /* do we need to notify anyone?*/
	 if (_notify!=null) {
	   _notify.menu_done();
	 }
       }

     } else {

       /* ok, we know someone is being moused on ... we may need
	to remove some of their children from the focus set */
       remove_children(copy_evt,mf);

       /* put events into coords of object we are dispatching to */
       copy_evt.global_to_local(mf);

       /* in any case we'll need to deliver the event to the object ...
	  is it mouse release*/
       if (copy_evt.id()==event.MOUSE_UP) {
	 /* its a mouse release */
	 mf.menu_release(copy_evt,user_info);

	 /* everyone will get notified to go out of the focus */
	 clear_focus(copy_evt);

	 /* we don't have anybody anymore */
	 most_recent=null;

	 /* do we need to notify anyone?*/
	 if (_notify!=null) {
	   _notify.menu_done();
	 }
       } else {
	 /* just send a mouse motion */
	 mf.menu_feedback(copy_evt,user_info);
       }
     }

     /* just return true to "fake_out" other policies/agents... . 
	this enforces that the menu interaction is modal */
     return true;
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Construct a new menu focus agent.
   */
  public menu_focus_agent() {
    super();
    /* not anybody activated yet */
    most_recent=null;
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function is used to find a focused on item.
   * If no focused on item is found, we return null, otherwise we
   * return the focused on item.
   * 
   * @param event evt the event we are checking
   * @return menu_focusable the object on which we should be focused or null
   */
  protected menu_focusable find_focused_item(event evt) {
    user_info_holder uih;
    menu_focusable mf;
    int i,index;
    
    /* generate the pick list */
    pick_collector pc=positional_policy_class.current_pick_list(evt);
    
    /* loop over the pick collector */
    for (i=0; i<pc.num_picks(); ++i) {
      /* grab the element and test to see if it is a menu_focusable
	 item ... if not, go to the next item */
      uih=pc.pick(i);
      
      /* is it of the right type? */
      if (uih.obj instanceof menu_focusable == false) continue;
      
      /* its ok to do this cast */
      mf=(menu_focusable)uih.obj;
      
      /* its menu_focusable, where is it on the list? */
      index=_focus_set.indexOf(mf);
      
      /* if its not in the list, then it doesn't matter */
      if (index<0) continue;
      
      /* its in the list, so return this object */
      return mf;
    }
    /* check to see if this is an interaction within the bounds
     * of the object which popped this menu up (if any) and
     * if wants to remove this menu */
    if (_notify!=null) {
      /* right now we don't need to know if they handled the event,
	 since they aren't over a menu this event ends up being discarded */
      handle_notify_position(evt);
    }

    /* never found anybody */
    return null;
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This is used to generate an exit event whenever a focused on item
   * is exited.  Note that you do NOT generate exit events unless the
   * object being exited is the farther one along in the focus list.
   * This is so that if you exit a menu item the wrong way (to the
   * left) and then sweep around the menu with the mouse and enter
   * the submenu, the submenu is there! 
   * 
   * @param event          evt        the event to dispatch
   * @param menu_focusable new_target the newly entered object (since this 
   *                                  object has been entered, we need to send 
   *                                  an exit event to the previously most 
   *                                  recent menu)
   * @param Object         user_info  the object passed to this agent when 
   *                                  this object entered the focus set
   */
  protected void generate_exit_event(event evt, menu_focusable new_target,
				    Object user_info) {

    /* if the object isn't at the end of the focus_set you
       don't send the message */

    /* if we have a recent object, send it exit */
    if ((most_recent!=null)  && 
	(_focus_set.elementAt(_focus_set.size()-1)==most_recent)) {
      most_recent.menu_exit(evt,user_info);
    }

    /* reset most recent */
    most_recent=new_target;
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This does the work of determining if we need to remove children from
   * the list of objects in the focus.
   * 
   * @param event          evt the event to be handled
   * @param menu_focusable mf  the object being checked for children who need 
   *                           to be removed
   */
  protected void remove_children(event evt, menu_focusable mf) {
    int i;
    menu their_child;
    event evtcopy=new event(evt);

    /* get the index of the object */
    int index=_focus_set.indexOf(mf); 

    /* if they are the last thing in the list, there are no children,
	 so do no work */
    if (index==_focus_set.size()-1) return;

    /* convert the event to their coordinate system */
    evtcopy.global_to_local(mf);

    /* ask what menu that they pop up */
    their_child = mf.menu_generates_submenu(evtcopy.local_x(), 
					    evtcopy.local_y());

    /* if their child is the one that is currently right after them,
	 then everything is cool */
    if (their_child==_focus_set.elementAt(index+1)) return;

    /* things are not cool ... blow away all the children */
    for (i=_focus_set.size()-1; i>index; --i) {
      /* do it in reverse order so people keeping state don't get confused */
      remove_from_focus((focusable)_focus_set.elementAt(i),evt);
    }
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * We override the set_focus_to method to make sure that if user
   * code calls this we don't end up with a bogus value of notify.
   * @param focusable to_obj    the object to set the focus to 
   * @param event     evt the   event causing the focus change
   * @param Object    user_info the object to be passed back to to_obj during 
   *                            subsequent calls
   */
  public void set_focus_to(focusable to_obj, event evt, Object user_info) 
{
    _notify=null;
    super.set_focus_to(to_obj,evt,user_info);
  }

   //had:
   //* @exception bad_value PROPAGATED
   //* @exception general PROPAGATED

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This is a special version of set_focus_to for the menu agent since
   * frequently menus are popped up from some object which needs to
   * be aware that the interaction has now ended. If you use this
   * interface, the menu_button passed as notify will get a call
   * on its menu_done() function to inform it that the interaction
   * is complete. 
   * @param focusable   to_obj    the object to set the focus to 
   * @param event       evt the   event causing the focus change
   * @param Object      user_info the object to be passed back to to_obj during 
   *                              subsequent calls
   * @param menu_notify notify    the object to be notified when the 
   *                              interaction ends or possibly changes state
   */
  public void set_focus_to(focusable to_obj, event evt, Object user_info,
			   menu_notifier notify) 
{
    _notify=notify;
    super.set_focus_to(to_obj, evt, user_info);
  }

   //had:
   //* @exception bad_value PROPAGATED
   //* @exception general PROPAGATED

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This is a function to allow us to determine if the object that popped
   * up this menu wants to and in fact has changed the menu out from under
   * us. 
   * @param event evt the event to deal with
   * @return boolean true if the notifier has put up a different menu
   */
  public boolean handle_notify_position(event evt) {
    int x,y;
    event evtcopy=new event(evt);

    /* convert the event to their coordinate system */
    evtcopy.global_to_local(_notify);

    /* bounds check against them */
    x=evtcopy.local_x();
    y=evtcopy.local_y();
    if ((x<0) ||
	(y<0) ||
	(x>_notify.w()) ||
	(y>_notify.h())) {

      /* they can't care, its not in their rectangle */
      return false;
    }

    /* they might care, call their function to check things out */
    return (_notify.menu_modify(x,y,evt));
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

}

/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
