package sub_arctic.lib;

import sub_arctic.output.drawable;
import sub_arctic.lib.top_level;
import sub_arctic.constraints.std_constraint_consts;

import java.applet.Applet;
import java.awt.Graphics;
import java.awt.Event;
import java.awt.Point;
import java.awt.Dimension;
import java.awt.Rectangle;

/**
 * This is the basic object which is used for creating applets using 
 * sub_arctic. This object is a subclass of Applet, so it can be used
 * in any browser.  <P>
 * 
 * The init method of this object is final.  This is so that you don't 
 * accidentally put in your own init() (which would be the normal thing to do 
 * when building an applet) and hence remove critical initializations
 * that the toolkit depends on for operation.  However, init() simply calls 
 * do_init(), and that is not final and can be overridden if absolutely 
 * necessary.<p>
 * 
 * Do_init() performs toolkit initialization and then calls the methods 
 * pre_build_ui(), build_ui() and post_build_ui() in that order. These are 
 * intended for data structure initialization, user interface construction,
 * and user_interface state manipulation respectively.  These are the routines
 * one would normally override when building a new subArctic applet.
 * 
 * @author Scott Hudson
 */
public class interactor_applet extends Applet implements std_constraint_consts {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Construct a new interactor_applet. Nothing to do since we use init() for 
   * initialization (once more of the AWT infrastructure has been initialized).
   */
  public interactor_applet()
    {
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Initialize by registering the component with the manager and calling 
   * the user routines: pre_build_ui(), build_ui(), and post_build_ui().
   * This function is final to prevent users from accidentally overriding it 
   * and getting very hosed. The real work is done in do_init() which is not 
   * final and can be overridden as needed.
   */
  public final void init()
    {
      do_init();
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Initialize by registering the component with the manager then calling
   * user routines. This does the real work for init() which is final to 
   * prevent users from accidentally overriding it and getting very hosed.  
   * This function calls four other functions: pre_build_ui(), 
   * make_top_level(), build_ui(top_level), and post_build_ui(top_level). 
   * The build_*() functions, and in particular the build_ui() function, is 
   * the primary place for UI construction, although you may want to do non-UI 
   * initialization in pre_build_ui(). Note that pre_build_ui() is called after 
   * make_toplevel(); this is useful if you wish to install constraints on the 
   * top_level before the user interface is constructed into it.
   */
  public void do_init()
    {
      manager.register_awt_component(this);
      top_level top;
      
      make_top_level();  // install it in the applet
      pre_build_ui();
      top=top_interactor();  
      build_ui(top);  // build the UI
      post_build_ui(top);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This user function is called to allow you to initialize data structures
   * that are needed for building the UI. By default, it does nothing.
   */
  public void pre_build_ui() {}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This user function is called to allow you to process the now 
   * constructed interactor tree after its built. 
   * 
   * @param base_parent_interactor t root of the newly constructed tree
   */
  public void post_build_ui(base_parent_interactor t) {}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function is called to create an install a top_level 
   * interactor in this applet. By default it creates a toplevel
   * of the same size as the interactor and then installs  it.
   */
  public void make_top_level() {
    top_level top=new top_level(0,0,size().width,size().height);
    set_top_interactor(top);
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function gets called so you can construct your user interface. The 
   * top_level provided is already installed as the root of your interactor 
   * tree for this applet. This is typically the main user routine for 
   * initializing the interface. By default, this function does nothing.
   *
   * @param base_parent_interactor top the new root interactor for this applet
   */
  public void build_ui(base_parent_interactor top) {}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /* Clean up when we are about to go away.  This unregisters the component
   * with the manager. 
   */
  public void destroy()
    {
      manager.unregister_awt_component(this);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
  /**
   * This is the top_level associated with this applet. 
   */
  protected top_level _top_interactor = null;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Fetch the top_level associated with this applet.
   *  
   * @return top_level the top_level interactor of this applet
   */
  public top_level top_interactor() {return _top_interactor;}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Set the top_level associated with this applet.
   * 
   * @param top_level top the new top_level interactor
   */
  public void set_top_interactor(top_level top)
    {
      _top_interactor = top;
      _top_interactor.set_awt_parent(this);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Remove the association between this applet and its top_level. Leaves
   * the applet with no current top_level.
   */
  public void remove_top_interactor()
    {
      _top_interactor.set_awt_parent(null);
      _top_interactor = null;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Part of the AWT infrastructure. This does a redraw of the applet 
   * by calling into manager to begin a drawing pass over the sub_arctic
   * interactor tree. 
   * 
   * @param Graphics g the graphics object to draw with.
   */
  public void update(Graphics g)
    {
      if (_top_interactor != null)
	{
	  manager.do_redraw(_top_interactor, new drawable(g));
	}
      else
       {
	 /* if there is no top level interactor, let the super class clear the
	  *  background for us.
	  */
	 super.update(g);
       }
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Part of the AWT infrastructure: This does a redraw of the applet 
   * by calling into manager to begin a drawing pass over the sub_arctic
   * interactor tree. <p>
   * 
   * This really should never get called (update() should be called 
   * instead), but apparently Netscape calls it direct in certain cases, 
   * so we just do the same as update (we don't just call update() since 
   * in the superclass, update() calls paint(), and update() can call 
   * super.update()).
   */
  public void paint(Graphics g)
    {
      if (_top_interactor != null)
	{
	  manager.do_redraw(_top_interactor, new drawable(g));
	}
      else
       {
	 /* if there is no top level interactor, let the super class do its
	  * thing.
	  */
	 super.paint(g);
       }
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Override AWT's reshape to catch size changes and propagate those to 
   * top level interactor. 
   * 
   * @param int x new X position of this applet
   * @param int y new Y position of this applet
   * @param int width new width of this applet
   * @param int height new height of this applet
   */
  public void reshape(int x, int y, int width, int height) 
    {
      Rectangle boundingBox;

      /* have the superclass to the reshape of the applet */
      super.reshape(x,y,width,height);

      /* try to set the new size */
      if (_top_interactor!=null) 
	{
	  /* set w/h if it has changed */
	  boundingBox = bounds();
	  if (_top_interactor.w() != boundingBox.width) 
	    _top_interactor.set_w(boundingBox.width);
	  if (_top_interactor.h() != boundingBox.height) 
	    _top_interactor.set_h(boundingBox.height);
	}
    }
  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
  /**
   * This is the AWT event handling entry point. All this does is
   * pass the event to the sub_arctic manager so it can being the
   * process of dispatching the input.
   *
   * @param Event evt the AWT (not sub_arctic) event to be handled
   * @return boolean true if we handled the event
   */
  public boolean handleEvent(Event evt)
    {
      /* test if we need a workaround of serious netscape bug */
      if (!manager.need_workaround(manager.WA_NS3_MAC,
				   manager.WA_REDUNDANT_MOVES))

	// normal version...

        /* hand event off to the overall toolkit manager to dispatch */
        return manager.dispatch_event(evt, top_interactor());

      // If we get here we need to do the netscape workaround...
      //
      // This workaround filters out adjacent move events that occur at the
      // same coordinates (which NS3.0 on the Mac seems to deliver in large
      // quantity).  
      //

      boolean result, this_is_move;

      /* is this the second move event in a row */
      this_is_move = evt.id == Event.MOUSE_MOVE || evt.id == Event.MOUSE_DRAG;
      if (this_is_move && last_evt_was_move)
        {
	  /* if its at the same position as the last move event bail out now */
          if (evt.x == last_move_evt_x && evt.y == last_move_evt_y)
          return true;
        }

       /* hand event off to the overall toolkit manager to dispatch */
       result = manager.dispatch_event(evt, top_interactor());
 
      /* if this was a move event, record the location for the next time */
      if (this_is_move)
        {
          last_evt_was_move = true;
          last_move_evt_x = evt.x;
          last_move_evt_y = evt.y;
        }
      else
       {
         last_evt_was_move = false;
       }

      return result;
    }

    /** (Netscape 3.0 workaround only): was the last event a move event? */
    protected static boolean last_evt_was_move   = false;

    /** (Netscape 3.0 workaround only): x position of last move event. */
    protected static int     last_move_evt_x = -9999;

    /** (Netscape 3.0 workaround only): y position of last move event. */
    protected static int     last_move_evt_y = -9999;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
