package sub_arctic.lib;
import sub_arctic.output.*;
import sub_arctic.constraints.std_function;
import sub_arctic.constraints.std_constraint_consts;

import java.applet.Applet;
import java.awt.Graphics;
import java.awt.Event;
import java.awt.Point;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Canvas;
import java.awt.Component;

/**
 * This is the "glue" which will stick a sub_arctic interface inside
 * an AWT interface. You can put a sub_arctic top_level inside
 * this object with set_top_level and you are in business.<p>
 * 
 * You have to provide a size for this object when you create it. If
 * you would like to derive a size from your sub_arctic interface, 
 * use interactor_canvas.size_from_top_level(). 
 *
 * @author Scott Hudson
 */
public class interactor_canvas extends Canvas {

  /** The width part of the size the user sets. */
  protected int _forced_width;

  /** The height part of the size the user sets. */
  protected int _forced_height;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Constructor an interactor_canvas.  This registers the component 
   * with the manager. 
   * 
   * @param int width the width of the canvas
   * @param int height the height of the canvas
   */
  public interactor_canvas(int width, int height)
    {
      /* the user always explicitly sets a size */
      _forced_width=width;
      _forced_height=height;
      
      manager.register_awt_component(this);
      resize(width,height);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Clean up when we are about to go away.  This unregisters the component
   * with the manager. 
   * 
   */
  public void finalize()
    {
      manager.unregister_awt_component(this);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This holds the top_level which is sitting inside this component. 
   */
  protected top_level _top_interactor = null;

  /**
   * Access the top_level inside this component.
   * 
   * @return top_level the top_level of this canvas
   */
  public top_level top_interactor() {return _top_interactor;}

  /**
   * Set the top_level associated with this canvas.
   *  
   * @param top_level top the new top_level for this canvas.
   */
  public void set_top_interactor(top_level top)
    {
      _top_interactor = top;
      _top_interactor.set_awt_parent(this);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
  /**
   * Remove the top_level from being used by this canvas. This will leave
   * this canvas with no top_level associated with it. 
   */
  public void remove_top_interactor()
    {
      _top_interactor.set_awt_parent(null);
      _top_interactor = null;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This is the redraw part of the AWT interface. This function begins
   * the drawing traversal of the sub_arctic interface.
   * 
   * @param Graphics g the graphics object to do the drawing on.
   */
  public void update(Graphics g)
    {
      if (_top_interactor != null)
	{
	  manager.do_redraw(_top_interactor, new drawable(g));
	}
      else
       {
	 /* if there is no top level interactor, let the super class clear the
	  *  background for us.
	  */
	 super.update(g);
       }
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This is the redraw part of the AWT interface. This function begins
   * the drawing traversal of the sub_arctic interface.
   *
   * This really should never get called (update() should be called 
   * instead), but apparently Netscape calls it direct in certain cases, 
   * so we just do the same as update (we don't just call update() since 
   * in the superclass update() calls paint(), and update() can call 
   * super.update()).
   * 
   * @param Graphics g the graphics object to do the drawing on.
   */
  public void paint(Graphics g)
    {
      if (_top_interactor != null)
	{
	  manager.do_redraw(_top_interactor, new drawable(g));
	}
      else
       {
	 /* if there is no top level interactor, let the super class do its
	  * thing.
	  */
	 super.paint(g);
       }
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * We override reshape to catch size changes and propagate those to top
   * level interactor. Thus, the top_level always is the same size and
   * shape as the surrounding component (this canvas).
   * 
   * @param int x the new X position of the object
   * @param int y the new Y position of the object
   * @param int w the new width of the object
   * @param int h the new height of the object
   */
  public void reshape(int x, int y, int width, int height) 
    {
      Rectangle boundingBox;

      /* have the superclass to the reshape of the applet */
      super.reshape(x,y,width,height);

      /* try to set the new size */
      if (_top_interactor!=null) 
	{
	  /* set w/h if it has changed */
	  boundingBox = bounds();
	  if (_top_interactor.w() != boundingBox.width) 
	    _top_interactor.set_w(boundingBox.width);
	  if (_top_interactor.h() != boundingBox.height) 
	    _top_interactor.set_h(boundingBox.height);
	}
    }
  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This is the AWT function which gets called when events happen. We
   * just redirect the event into the manager which ends up calling 
   * all of the sub_arctic input dispatch machinery.
   *
   * @param Event evt the AWT event to process (not the sub_arctic event)
   * @return true if we handled the event
   */
  public boolean handleEvent(Event evt)
    {
      /* hand event off to the overall toolkit manager to dispatch */
      return manager.dispatch_event(evt, top_interactor());
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function is part of the AWT world. It appears to be called 
   * immediately after the peer of this object is created. 
   * Its not entirely clear to me what this code <I>should</I> do.
   */
  public void addNotify() {
    super.addNotify();
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function is part of the AWT world. It is supposed to be called
   * by layout manager's to ask us how big we would like to be. We just
   * tell it our minimum size. 
   * 
   * Its not clear when, where, why, or if this function will actually 
   * be called by a layout manager. If you put an interactor_canvas 
   * in a frame it clearly is <STRONG>not</STRONG> called.
   * 
   * @return Dimension the dimensions we prefer for this canvas
   */
  public Dimension preferredSize() { 
    return (minimumSize());
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function is part of the AWT world. It is supposed to be called
   * by layout manager's to ask how what the minimal amount of screen
   * space we can live with is.We return the size that the user initially
   * gave us when this object was created.
   * 
   * Its not clear when, where, why, or if this function will actually 
   * be called by a layout manager. If you put an interactor_canvas 
   * in a frame it clearly is <STRONG>not</STRONG> called.
   *
   * @return Dimension the minimum dimension we can live with
   */
  public Dimension minimumSize() { 
    Dimension d=new Dimension(_forced_width,  _forced_height);
    return d;
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * From a toplevel, determine a dimension. This can be used for sizing
   * a canvas (or frame) to your toplevel's size. This is especially
   * useful for inside-out layouts. 
   * 
   * Note: If you use this function and your toplevel has constraints
   * on it, they will be removed. This is because toplevel's sizes are
   * connected (directly) to the canvas/applet in which they live.
   * Thus, if the user resized your window, they would get an error
   * for assigning to a constrained value. Thus, you can look at 
   * constraints on a top_level as a "one-shot" deal; use them 
   * to calculate your interface's initial size and then they go away. 
   * 
   * @param top_level tl the top_level whose size you want
   * @return Dimension the size (width/height) of this canvas.
   */
  public static Dimension size_from_top_level(top_level tl) {
    int cache_w=tl.w(), cache_h=tl.h();
    if (!tl.w_constraint().is_none()) {
      tl.set_w_constraint(std_constraint_consts.NO_CONSTRAINT);
    }
    if (!tl.h_constraint().is_none()) {
      tl.set_h_constraint(std_constraint_consts.NO_CONSTRAINT);
    }
    return new Dimension(cache_w, cache_h);
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}

/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
