package sub_arctic.lib;

/**
 * This interface is the API that listboxes use to communicate
 * with their elements. Any object that implements this interface may be
 * placed into a listbox.
 * @author Ian Smith
 */
public interface list_element extends interactor {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This method will be called by the listbox to inform the
   * object that it should highlight. This does not (necessarily)
   * imply that the user clicked on this object, as this selection
   * might be programmatic or the result of user actions which
   * select a group of objects.  <P>
   * 
   * Implementations of this interface can assume that are not
   * initially highlighted.
   * 
   */
  public void highlight();

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This method will be called by the listbox to inform the
   * object that it should no longer be highlighted.
   */
  public void unhighlight();

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This method is called by the listbox to try to get a more
   * convenient representation of the list_element for the
   * user-level code. If you return an object here that object
   * will be placed in the "selected set" of objects when the
   * user-level code asks for the set of selected objects or in
   * the vector of "contents" when the user asks for that. If you
   * return null here, the object implementing this interface
   * will be placed in the set of selections or contents.<P>
   * 
   * This function is here to allow objects which have simple
   * semantics to export those semantics to user-level code. For
   * example, an implementation of this interface which displays
   * text strings will want to return the string displayed 
   * from this function, rather than the object which does the
   * display.
   * 
   * @return Object the object which this object can be converted to for 
   *                simplifying user-level code.
   */
  public Object convert_to_object();

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This returns the current highlight status of the object.
   */
  public boolean highlighted();

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This method is used to inform the object that it has become the
   * focus of a listbox. Listbox elements should draw themselves 
   * differently to indicate this demarcation. Listbox elements may
   * be the focus when they are not highlighted and vice-versa.
   */
  public void become_focus();

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This method is used to inform the object that it is not
   * the focus any longer.
   */
  public void lost_focus();

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This method is used to query the object about whether or
   * not it is the focus.
   * @return boolean true if this object is the focus
   */
  public boolean focused(); 

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This should return the preferred width of this item.  The item
   * must be able to display itself at any width, however. This is
   * used by the parent to determine which list element is widest
   * and force all the children to be that width.
   * 
   * @return int the object's preferred width in pixels
   */
  public int preferred_width();

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}

/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
