
package sub_arctic.lib;

import java.awt.Rectangle;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Color;

/** 
 * An interface_pred predicate class that performs a semantic redraw action
 * for a semantic lens.  In this case the object draws a small name tag 
 * containing the class name, and a bounding box.  This predicate expects
 * a sem_draw_context object as its parameters argument.  This predicate is
 * executed for its side effect only and always returns false.
 *
 * @see sub_arctic.lib.base_interactor#traverse_and_collect
 * @author Scott Hudson
 */
public class nametag_sem_draw implements interactor_pred {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Unique id for this drawing traversal */
  public static final int id = manager.unique_int();

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Font to draw class tags in. */
  protected Font _font = new Font("Helvetica",Font.PLAIN,9);

  /** 
   * Font to draw class tags in. 
   * @return Font the font to draw in.
   */
  public Font font() {return _font;}

  /** 
   * Set font to draw class tags in.
   * @param Font fnt the font to draw in. 
   */
  public void set_font(Font fnt) 
    {
      _font = fnt;
      metrics = manager.get_metrics(_font);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** FontMetrics object for the current font */
  protected FontMetrics metrics; 

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Tag color */
  protected Color _tag_color = new Color(243, 243, 243);

  /** 
   * Tag color.
   * @return Color the color we draw tags in.
   */
  public Color tag_color() {return _tag_color;}

  /** 
   * Set tag color.
   * @param Color c the color we draw tags in.
   */
  public void set_tag_color(Color c) {_tag_color = c;}


  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Full constructor. 
   * @param Font fnt the font to draw tags in.
   * @param Color c the color to draw tags in.
   */
  public nametag_sem_draw(Font fnt, Color c)
    {
      /* establish color */
      set_tag_color(c);

      /* establish font and metrics object */
      set_font(fnt);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Constructor using default font and tag color. */
  public nametag_sem_draw()
    {
      /* use already initialized font to also set up metrics object */
      set_font(_font);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Perform the predicate test.  In this case we are doing this for the side
   * effect (drawing a tag and bounding box for the object) and always return
   * false.
   *  
   * @param obj        the interactor the predicate is drawing.
   * @param parameters a sem_draw_context object containing the a drawable 
   *                   object and other bookkeeping needed for drawing.
   * @return boolean false in all cases.
   */
  public boolean test(interactor obj, Object parameters) 
    {
      sem_draw_context context;
      String           tag;
      Rectangle        bound;
      int              sz_w, sz_h, last_dot;

      /* type check the parameters */
      if (!(parameters instanceof sem_draw_context))
	throw new sub_arctic_error("Non-sem_draw_context object "+
				   "passed as parameter");

      /* don't draw semantic lens or semantic lens parent objects */
      if (obj instanceof semantic_lens || obj instanceof semantic_lens_parent) 
        return false;

      /* extract drawing context */
      context = (sem_draw_context)parameters;

      /* don't draw if we are trivially rejected */
      bound = new Rectangle(0,0,obj.w(), obj.h());
      if (!context.surface.getClipRect().intersects(bound))
	{
	  return false;
	}

      /* draw the bounding rectangle for the object */
      context.surface.setColor(Color.red);
      context.surface.drawRect(0, 0, obj.w()-1, obj.h()-1);

      /* extract the type tag and and strip off leading package names */
      tag = obj.getClass().getName();
      last_dot = tag.lastIndexOf('.');
      if (last_dot != -1) tag = tag.substring(last_dot+1);

      /* measure it */
      sz_w = metrics.stringWidth(tag) + 4;
      sz_h = metrics.getHeight() + 2;

      /* draw the text tag */
      context.surface.setColor(tag_color());
      context.surface.fillRect(0, 0, sz_w-1, sz_h-1);
      context.surface.setColor(Color.black);
      context.surface.setFont(font());
      context.surface.drawString(tag, 2, metrics.getAscent()+1);
      context.surface.drawRect(0,0, sz_w-1, sz_h-1);

      /* don't actually add the object to the result collection */
      return false;
    }

   //had:
   //* @exception sub_arctic.exception.bad_value thrown if the parameters 
   //*    argument is not a sem_draw_context object.

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
};
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
