package sub_arctic.lib;

import sub_arctic.lib.sub_arctic_error;
import sub_arctic.lib.manager;
import sub_arctic.output.drawable;
import sub_arctic.output.style_manager;
import sub_arctic.output.style;
import sub_arctic.output.color_pair;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Color;

/** 
 * Single line text displayer.  This class displays a single line of text 
 * with selection point or area in a fashion suitable for a text edit 
 * interactor.   Selection positions in strings refer to the
 * spaces before the characters (numbered from 0).  So for example a
 * selection from 0 to 0 places the cursor before the first character
 * (the character at index 0) while a selection from 1 to 2 selects the
 * second character (the character at index 1).  
 * 
 * @author Scott Hudson
 */
public class oneline_text_display extends base_interactor {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Vertical spacing around the text (both top and bottom have 
   this space) 
   */
  protected int _v_spacing = 3;

  /**
   * This retrieves the vertical spacing of this interactor.
   * @return int the amount of space on the right and left of this interactor
   */
  public int v_spacing() { return _v_spacing;}

  /**
   * Set the vertical spacing of this interactor.
   * @param int n new amount of vertical space.
   */
  public void set_v_spacing(int n) { 
    _v_spacing=n;
    /* compute the height of the object based on the font */
    set_intrinsic_h(_metric.getHeight()-_metric.getLeading()+(2*_v_spacing));

    /* redraw us */
    damage_self();
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Horizontal spacing */
  protected int _h_spacing = 3;

  /**
   * This retrieves the horizontal spacing of this interactor.
   * @return int the amount of space on the right and left of this interactor
   */
  public int h_spacing() { return _h_spacing;}

  /**
   * Set the horizontal spacing of this interactor.
   * @param int n new amount of horizontal space.
   */
  public void set_h_spacing(int n) { 
    _h_spacing=n;
    // recalculate the width
    set_w(_metric.stringWidth(text()) + 2*_h_spacing);
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Cached font metrics object for the current font */
  protected FontMetrics _metric;

  /** Font that interactor displays in */
  protected Font _font;

  // note: later we might just store the metrics object, since we can get the
  // font from that

  /** 
   * Font that interactor displays in... if you don't set the
   * font, you'll get the system's default font. 
   * @return Font the font this object is using
   */
  public Font font() {
    if (_font==null) {
      return style_manager.default_font();
    }
    return _font;
  }
  
  /** 
   * Set font that interactor displays in 
   * @param Font fnt the new font (use null for the system default font)
   */
  public void set_font(Font fnt) {
      /* if they pass a null font we use the default */
      if (fnt == null) {
	fnt = style_manager.default_font();
      }
      /* if this is not a change, bail out early */
      if (fnt.equals(_font)) return;

      /* store the font and stash a metric object for it */
      _font = fnt;
      _metric = manager.get_metrics(_font);

      /* compute the height of the object based on the font */
      set_intrinsic_h(_metric.getHeight()-_metric.getLeading()+2*_v_spacing);

      /* redraw us */
      damage_self();
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
  /**
   * The color pair currently being used to display the text and 
   * background of the object.
   */
  protected color_pair _colors=null;
  /**
   * This function retreives the currently color_pair being used for
   * displaying this objects foreground and background colors. 
   * @return color_pair the color pair currently in use (foreground color 
   *         is the color of the text)
   */
  public color_pair colors() { 
    if (_colors==null) {
      _colors=new color_pair(style_manager.default_color_scheme().
			     foreground(),
			     style_manager.default_color_scheme().
			     text_background());
    }
    return _colors;
  }
  /**
   * You can call this function to set a pair of colors that are used
   * to display the text background and the text of this object. The
   * foreground color of the color_pair is the color of the text.
   * If you set the color pair in use to null, the system will use
   * the default system colors.
   * @param color_pair p the new color pair to use for this interactor (null
   *                   means to use the system defualt colors)
   */
  public void set_colors(color_pair p) {
    _colors=p;
    damage_self();
  }
  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** String currently being displayed. */
  protected String _text = "";

  /** 
   * String currently being displayed. 
   * @return String the string that is being displayed
   */
  public String text() {return _text;} 

  /** 
   * Set string displayed. 
   * @param String text the new string
   */
  public void set_text(String txt) 
    {
      /* convert null to empty string */
      if (txt == null) txt = "";

      /* save the string and cause a redraw */
      _text = txt;
      damage_self();
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Adjustment factor to keep the current selection (or part of it) inside
   * the bounds of the object.  This is set by ensure_inside(). 
   */
  protected int _x_adjust = 0;

  /** 
   * Adjustment factor to keep the current selection (or part of it) inside
   * the bounds of the object.  This is set by ensure_inside(). 
   */
  public int x_adjust() {return _x_adjust;}

  /** 
   * Compute the adjustment needed to make sure the given selection position
   * (a character position, not a pixel position) is inside the object's 
   * display.  This routine gets called automatically when a single position
   * selection is established.  For a range selection, it must be called 
   * "manually" to indicate which end of the selection should be inside.<p> 
   * 
   * @param int pos selection position that should be inside the display.
   */
  public void ensure_inside(int pos) 
    {
      int pixel_pos;

      /* make sure the position is in range */
      if (pos < 0) pos = 0;
      if (pos > text().length()) pos = text().length();

      /* compute the offset from the start of the string to the position */
      pixel_pos = _metric.stringWidth(text().substring(0,pos));

      /* change x_adjust if that is outside */
      if (_x_adjust + pixel_pos < _h_spacing)
	{
	  _x_adjust = - pixel_pos; // h_spacing goes in in draw_self_local()
	  damage_self();
	}
      else if (_x_adjust + pixel_pos > w() - 2*_h_spacing)
	{
	  _x_adjust = (w() - 2*_h_spacing) - pixel_pos; 
	  damage_self();
	}
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Constant for a special selection position used to denote that there 
   * is no selection. 
   */
  public static final int NO_SELECTION = -1;

  /** Constant used to denote that the selection as at the end of the text. */
  public static final int SELECT_END = -2;

  /** 
   * Start of current selection.  The special value NO_SELECTION denotes
   * the fact that there is no current selection.  Note: selection positions 
   * denote the space before the character at the given ordinal position.  
   * Consequently, 0 denotes the position before the beginning of the string, 
   * and text().length() denotes the position after the last character.
   */
  protected int _start_of_selection = 0; 

  /** 
   * Start of current selection.  The special value NO_SELECTION denotes
   * the fact that there is no current selection.  Note: selection positions 
   * denote the space before the character at the given ordinal position.  
   * Consequently, 0 denotes the position before the beginning of the string, 
   * and text().length() denotes the position after the last character.  
   * @return the start of the selection or a special constant
   */
  public int start_of_selection() {return _start_of_selection;}

  /** 
   * End of current selection.  The special value NO_SELECTION denotes
   * the fact that there is no current selection.  Note: selection positions 
   * denote the space before the character at the given ordinal position.  
   * Consequently, 0 denotes the position before the beginning of the string, 
   * and text().length() denotes the position after the last character.
   * @return the end of the selection or a special constant
   */
  protected int _end_of_selection = 0; 

  /** 
   * End of current selection.  The special value NO_SELECTION denotes
   * the fact that there is no current selection.  Note: selection positions 
   * denote the space before the character at the given ordinal position.  
   * Consequently, 0 denotes the position before the beginning of the string, 
   * and text().length() denotes the position after the last character.
   * @return the end of the selection or a special constant
   */
  public int end_of_selection() {return _end_of_selection;}

  /** 
   * Set the current selection.  The special value NO_SELECTION is used 
   * to denote the fact that there is no current selection.  If this value
   * is applied to either the start or end of the selection, both values
   * will be set to NO_SELECTION.  The special value SELECT_END denotes a
   * the position after the last character of the string (which will be 
   * computed and stored).  Note: selection positions denote the space
   * before the character at the given ordinal position.  Consequently,
   * 0 denotes the beginning of the string, and text().length() denotes 
   * the position after the last character. 
   * 
   * @param int start_loc the starting location of the selection
   * @param int end_loc the ending location of the selection
   */
  public void set_selection(int start_loc, int end_loc)
    {
      int last_pos = text().length();

      /* if we have no selection, we set it on both ends */
      if (start_loc == NO_SELECTION) end_loc = NO_SELECTION;
      if (end_loc == NO_SELECTION) start_loc = NO_SELECTION;

      /* replace SELECT_END special value */ 
      if (start_loc == SELECT_END) start_loc = text().length();
      if (end_loc == SELECT_END)   end_loc   = text().length();

      /* make sure we are ordered properly */
      if (end_loc < start_loc)
	{
	  int temp = end_loc; 
	  end_loc = start_loc; 
	  start_loc = temp;
	}

      /* don't go past the end or before the beginning */
      if (start_loc > last_pos) start_loc = last_pos;
      if (end_loc > last_pos)   end_loc = last_pos;
      if (start_loc < 0 && start_loc != NO_SELECTION) start_loc = 0;
      if (end_loc < 0   && end_loc != NO_SELECTION)   end_loc = 0;

      /* do the assignment and ask for redraw if this is a change */
      if (start_loc != _start_of_selection || end_loc != _end_of_selection)
	{
	  _start_of_selection = start_loc;
	  _end_of_selection = end_loc;

	  /* if we have a single point selection, make sure its displayed */
	  if (start_loc != NO_SELECTION && start_loc == end_loc)
	    ensure_inside(start_loc);

	  damage_self();
	}
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Constant used to store boxed flag within interactor flags */
  public static final int BOXED = FIRST_FREE_FLAG;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Does the interactor have a box drawn around it 
   * @return boolean true if the object does have a box around it
   */
  public boolean boxed()
    {
      return flag_is_set(BOXED);
    }

  /** 
   * Set the flag indicating whether interactor is drawn with a surrounding 
   *  box. 
   * @param boolean bxv true if you want a box around this object, false 
   *                    otherwise
   */
  public void set_boxed(boolean bxv)
    {
      /* if we have a change set the bit and ask for a redraw */
      if (bxv != boxed())
	{
	  set_flag_bit(BOXED, bxv);
	  damage_self();
	}
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Full constructor with explicit width  
   * @param int     xv       the x coordinate of this object
   * @param int     yv       the y coordinate of this object
   * @param int     wv       the width of this object
   * @param String  init_str the initial string to display
   * @param Font    fnt      the font to use
   * @param boolean boxed    true if you want a box around the object
   */
  public oneline_text_display(
    int              xv, 
    int              yv, 
    int              wv,
    String           init_str, 
    Font             fnt, 
    boolean          boxed)
    {
      super(xv,yv,wv,10/* real h gets computed in set_font() */);

      set_font(fnt);
      set_text(init_str);
      set_boxed(boxed);
      set_selection(NO_SELECTION, NO_SELECTION);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Full constructor where width is determined by initial string 
   * @param int     xv       the x coordinate of this object
   * @param int     yv       the y coordinate of this object
   * @param String  init_str the initial string to display
   * @param Font    fnt      the font to use
   * @param boolean boxed    true if you want a box around the object
   */
  public oneline_text_display(
    int              xv, 
    int              yv, 
    String           init_str, 
    Font             fnt, 
    boolean          boxed)
    {
      super(xv,yv,10/*w set below */,10/* real h computed in set_font() */);

      set_font(fnt);
      set_text(init_str);
      set_boxed(boxed);
      set_selection(NO_SELECTION, NO_SELECTION);

      /* compute the width from the string */
      set_w(_metric.stringWidth(text()) + 2*_h_spacing);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Constructor with default font and boxed status but explicit position
   * 
   * @param int    xv       the x coordinate value of this object
   * @param int    yv       the y coordinate value of this object
   * @param String init_str the initial string to display
   */
  public oneline_text_display(
    int              xv, 
    int              yv, 
    String           init_str)
    {
      this(xv, yv, init_str, null, true);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Constructor with all the defaults and a size which we
   * assume you are going to control either control explicitly
   * or via constraints.
   * 
   * @param the initial string to display
   */
  public oneline_text_display(String init_str)
    {
      this(0, 0, init_str, null, true);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Declare that our height is intrinsically constrained 
   * @return int the constant H so everyone knows we have intrinsic height.
   */
  public int intrinsic_constraints() { return H; }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Draw the display. 
   * @param drawable d the surface to draw this object on
   */
  protected void draw_self_local(drawable d) 
{
      int x_pos, y_pos, wid, sel_pos;
      String front = null;
      String selection = null;
      String back = null;
      style cs=style_manager.current_style();
      int dvs=cs.drawable_vertical_space(),
	dhs=cs.drawable_horizontal_space();
      color_pair pair=colors();

      /* if there is enough space, use the style routines */
      if ((_v_spacing>=dvs) &&
	  (_h_spacing>=dhs)) {

	/* there is enough */
	cs.drawable_prepare_rect(d,0,0,w(),h(),false, true);

	/* use the background color for the background */
	d.setColor(pair.background());

	/* fill in the rect, but be sure to not go over the beveling */
	d.fillRect(dhs,dvs,w()-(2*dhs),h()-(2*dvs));

      } else {
	/* just fill the background, no bevelling */
	d.setColor(pair.background());
	d.fillRect(0,0,w()-1,h()-1);

      }

      /* set font to our font, and start drawing in foreground */
      d.setFont(font());
      d.setColor(pair.foreground());

      /* is there text to draw? */
      if ((_text==null) || (_text.equals(""))) {
	/* might be boxed?*/
	if (boxed())
	  d.drawRect(0,0,w()-1,h()-1);
	return;
      }

      /* extract parts of string (empty parts are left null) */
      if (start_of_selection() == NO_SELECTION)
	{
	  front = text();
	}
      else
	{
	  if (start_of_selection() > 0)
	    front = text().substring(0,start_of_selection());
	  if (end_of_selection() - start_of_selection() > 0)
	    selection=text().substring(start_of_selection(),end_of_selection());
	  if (end_of_selection() < text().length())
	    back = text().substring(end_of_selection(), text().length());
	}

      /* draw the parts of the string */
      x_pos = _h_spacing + x_adjust();
      y_pos = _v_spacing+_metric.getAscent();
      sel_pos = x_pos;

      if (front != null)
	{
	  /* draw the front text in black */
          d.drawString(front, x_pos, y_pos);
	  x_pos += _metric.stringWidth(front);
          sel_pos = x_pos;
	}

      if (selection != null)
	{
	  /* fill the selection area with black */
	  wid = _metric.stringWidth(selection); 
	  d.fillRect(x_pos,_v_spacing,wid,h()-2*_v_spacing);

	  /* draw text over that in other color */
          d.setColor(pair.background());
          d.drawString(selection, x_pos, y_pos);

	  /* set up for rest of drawing */
          d.setColor(pair.foreground());
	  x_pos += wid;
	}

      if (back != null)
	{
	  /* draw the final part of the text in black */
          d.drawString(back, x_pos, y_pos);
	}

      /* if we have zero size selection draw as a line */
      if (start_of_selection() != NO_SELECTION && 
	  start_of_selection() == end_of_selection())
	{
	  d.drawLine(sel_pos,_v_spacing,sel_pos,h()-_v_spacing);
	}

      /* draw box if requested */
      if (boxed())
        d.drawRect(0,0,w()-1,h()-1);
    }

   //had:
   //* @exception general PROPAGATED

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
