package sub_arctic.lib;

import sub_arctic.lib.oneline_text_display;
import sub_arctic.lib.manager;
import sub_arctic.output.color_pair;
import sub_arctic.output.drawable;
import sub_arctic.input.event;
import sub_arctic.input.callback_object;
import sub_arctic.anim.trajectory;
import sub_arctic.anim.transition;
import sub_arctic.anim.animatable;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Color;
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.Hashtable;

/**
 * This is a subclass of sliding_text which adds text style and color changes
 * to the text.  Here we support HTML-like tags within the text which indicate
 * style and color changes.  Tags accepted include: &lt;b&gt;...&lt;/b&gt; 
 * and &lt;i&gt;...&lt;/i&gt; for bold and italics, as well as matching tag 
 * pairs for the colors: black, blue, cyan, darkGray, gray, green, lightGray, 
 * magenta, orange, pink, red, white, and yellow (e.g., 
 * &lt;red&gt;...&lt;/red&gt;).
 * 
 * @see sliding_text
 * @author Scott Hudson
 */
public class sliding_stylized_text extends sliding_text {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Make an object using some simple defaults. It doesn't try to position 
   * itself. It assumes you'll do this with constraints. It sizes height by
   * the size of the text and width is set to a default.
   *
   * @param String          s  the string to put in the display
   * @param callback_object cb object to call back at end of scroll
   */
  public sliding_stylized_text(String s, callback_object cb) {
    super(s,cb);
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Again, make an object with some defaults. This time you supply the
   * width. Assumes x and y will use constraints. 
   * 
   * @param String s the string to display 
   * @param int width the width of the displayable area in pixels
   * @param callback_object cb object to call back at end of scroll
   */
  public sliding_stylized_text(String s, int width, callback_object cb) {
    super(s,width,cb);
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Again, make an object with some defaults. This time you supply the
   * width & a font. Assumes x and y will use constraints.
   * 
   * @param String s the displayed string
   * @param int width the number of pixels wide the display area is
   * @param Font f the font render the string in
   * @param callback_object cb object to call back at end of scroll
   */
  public sliding_stylized_text(String s, int width, Font f, callback_object cb)
    {
      super(s,width,f,cb);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Again, make a label with some defaults. This time you supply the
   * font. Assumes x and y will use constraints.
   *  
   * @param String s the displayed string
   * @param Font f the font to use for rendering the string
   * @param callback_object cb object to call back at end of scroll
   */
  public sliding_stylized_text(String s, Font f, callback_object cb) {
    super(s,f,cb);
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Helper routine to clone a font in a different style. 
   *
   * @param Font fnt the font to clone
   * @param int  new_style new style bits -- these should be a combination of
   *                       Font.PLAIN, Font.Bold, and font.ITALIC.
   */
  protected static Font restyle(Font fnt, int new_style)
    {
      return new Font(fnt.getName(), new_style, fnt.getSize());
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Vector of effects that are being applied to the text. This vector
   * contains text_effect_record objects each of which encode a substring
   * of text that they apply to, a color and a font.  Effect records must
   * not overlap and must be in sorted order.
   */
  protected Vector _effect_list;

  /** 
   * Vector of effects that are being applied to the text. This vector
   * contains text_effect_record objects each of which encode a substring
   * of text that they apply to, a color and a font.  Effect records must
   * not overlap and must be in sorted order.
   * @return Vector containing text_effect_record objects to apply to the text.
   */
  public Vector effect_list() {return _effect_list;}

  /** First effect record that could affect the displayed text (i.e. has a 
   *  start position past first_vis_char.  If there is no such record, 
   *  this is set past the last index of the effect_list vector.
   */
  protected int first_vis_effect = 0;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Hashtable holding codes for the tags we know how to process.  Two types 
   *  of entries get put in this table: Color objects representing the fact 
   *  that the tag represents a color change, and Integer objects which encode
   *  add bold with 1, remove bold with -1, add italics with 2, remove 
   *  italics with -2 and revert to normal color with 0.
   */
  protected static Hashtable tag_table = null;

  /** 
   * Initialize the hashtable holding codes for the tags we know how to 
   * process.  
   */
  protected void setup_tag_table()
    {

      tag_table = new Hashtable();

      tag_table.put ("b", new Integer(1));
      tag_table.put ("B", new Integer(1));
      tag_table.put ("/b", new Integer(-1));
      tag_table.put ("/B", new Integer(-1));
      tag_table.put ("i", new Integer(2));
      tag_table.put ("I", new Integer(2));
      tag_table.put ("/i", new Integer(-2));
      tag_table.put ("/I", new Integer(-2));

      tag_table.put("black", Color.black); 
      tag_table.put("/black", new Integer(0)); 
      tag_table.put("blue", Color.blue); 
      tag_table.put("/blue", new Integer(0));
      tag_table.put("cyan", Color.cyan); 
      tag_table.put("/cyan", new Integer(0)); 
      tag_table.put("darkGray", Color.darkGray); 
      tag_table.put("/darkGray", new Integer(0)); 
      tag_table.put("gray", Color.gray); 
      tag_table.put("/gray", new Integer(0)); 
      tag_table.put("green", Color.green); 
      tag_table.put("/green", new Integer(0)); 
      tag_table.put("lightGray", Color.lightGray); 
      tag_table.put("/lightGray", new Integer(0)); 
      tag_table.put("magenta", Color.magenta); 
      tag_table.put("/magenta", new Integer(0)); 
      tag_table.put("orange", Color.orange); 
      tag_table.put("/orange", new Integer(0)); 
      tag_table.put("pink", Color.pink); 
      tag_table.put("/pink", new Integer(0)); 
      tag_table.put("red", Color.red); 
      tag_table.put("/red", new Integer(0));
      tag_table.put("white", Color.white); 
      tag_table.put("/white", new Integer(0));
      tag_table.put("yellow", Color.yellow); 

      tag_table.put("BLACK", Color.black); 
      tag_table.put("/BLACK", new Integer(0)); 
      tag_table.put("BLUE", Color.blue); 
      tag_table.put("/BLUE", new Integer(0));
      tag_table.put("CYAN", Color.cyan); 
      tag_table.put("/CYAN", new Integer(0)); 
      tag_table.put("darkgray", Color.darkGray); 
      tag_table.put("DARKGRAY", Color.darkGray); 
      tag_table.put("/DARKGRAY", new Integer(0)); 
      tag_table.put("/darkgray", new Integer(0)); 
      tag_table.put("GRAY", Color.gray); 
      tag_table.put("/GRAY", new Integer(0)); 
      tag_table.put("GREEN", Color.green); 
      tag_table.put("/GREEN", new Integer(0)); 
      tag_table.put("LIGHTGRAY", Color.lightGray); 
      tag_table.put("lightgray", Color.lightGray); 
      tag_table.put("/LIGHTGRAY", new Integer(0)); 
      tag_table.put("/lightgray", new Integer(0)); 
      tag_table.put("MAGENTA", Color.magenta); 
      tag_table.put("/MAGENTA", new Integer(0)); 
      tag_table.put("ORANGE", Color.orange); 
      tag_table.put("/ORANGE", new Integer(0)); 
      tag_table.put("PINK", Color.pink); 
      tag_table.put("/PINK", new Integer(0)); 
      tag_table.put("RED", Color.red); 
      tag_table.put("/RED", new Integer(0));
      tag_table.put("WHITE", Color.white); 
      tag_table.put("/WHITE", new Integer(0));
      tag_table.put("YELLOW", Color.yellow); 
      tag_table.put("/YELLOW", new Integer(0));

    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Process a string with HTML-like tags to produce a new string without
   * the tags and an effect list corresponding to the tags.  Tags accepted
   * include: &lt;b&gt;...&lt;/b&gt; and &lt;i&gt;...&lt;/i&gt; for bold and 
   * italics, as well as matching tag pairs for the colors: black, blue,
   * cyan, darkGray, gray, green, lightGray, magenta, orange, pink, red,
   * white, and yellow (e.g., &lt;red&gt;...&lt;/red&gt;).
   * 
   * @param String tagged_string the input string with tags
   * @param Vector result_eff    a Vector object that will be filled with 
   *                             text_effect_records corresponding to the 
   *                             tags found.
   * @return String new string with tags stripped out
   */
  protected String parse_tags(String tagged_string, Vector result_eff)
    {
      StringBuffer    result = new StringBuffer();
      StringTokenizer tag_seq = new StringTokenizer(tagged_string,"<>", true);
      boolean         inside = false;
      String          token, tag_word = "";
      Object          word_lookup;
      int             val;
      int             pos = 0;
      int             effect_start = 0;
      Color           cur_color = null;
      int             cur_font = Font.PLAIN;

      /* clear our result effect list */
      result_eff.removeAllElements();

      /* make sure we have our lookup table */
      if (tag_table == null) setup_tag_table();

      while (tag_seq.hasMoreTokens())
	{
	  /* get the next hunk of text */
	  token = tag_seq.nextToken();

	  /* Look for opener */
	  if (token.equals("<")) 
	    {
	      /* we go inside a tag, or this is a bad tag */
	      if (!inside) 
		{
		  inside = true;
		  tag_word = "";
		}
	      else
		{
		  result.append("<");
		  result.append(tag_word);
	          result.append(token);
		  pos += 1 + tag_word.length() + token.length();
		  inside = false;
		  tag_word = "";
		}
	    }
	  /* look for closer */
	  else if (token.equals(">"))
	    {
	      /* if this properly terminates a tag, process it */
	      if (inside)
		{

		  /* look up the tag */
		  word_lookup = tag_table.get(tag_word);
		  if (word_lookup == null)
		    {
		      /* not found so put the tag out as text */
		      result.append("<");
		      result.append(tag_word);
		      result.append(">");
		      pos += 2 + tag_word.length();
		    }
		  else if (word_lookup instanceof Color)
		    {
		      /* end current effect region */
		      if ((cur_color != null || cur_font != Font.PLAIN) &&
			  pos >= effect_start)
			{
			  result_eff.addElement(new text_effect_record(
				      effect_start,pos,cur_font, cur_color));
			}

		      /* set up for the next */
		      cur_color = (Color)word_lookup;
		      effect_start = pos;
		    }
		  else if (word_lookup instanceof Integer)
		    {
		      /* end current effect region */
		      if ((cur_color != null || cur_font != Font.PLAIN) &&
			  pos >= effect_start)
			{
			  result_eff.addElement(new text_effect_record(
				      effect_start,pos,cur_font, cur_color));
			}

		      /* set up for the next */
                      val = ((Integer)word_lookup).intValue();
		      if (val == 0)
			cur_color = null;
		      else if (val == -1)
			cur_font &= ~Font.BOLD;
		      else if (val == 1)
			cur_font |= Font.BOLD;
		      else if (val == -2)
			cur_font &= ~Font.ITALIC;
		      else if (val == 2)
			cur_font |= Font.ITALIC;
		      effect_start = pos;
		    }
		  tag_word = "";
		  inside = false;
		}
	      else
		{
		  /* just a stray */
		  result.append(token);
		  pos += token.length();
		}
	    }
	  /* handle everything else */
	  else 
	    {
	      /* if we have seen the opener keep the word */
	      if (inside) 
		tag_word = token;
	      else
		{
		  /* otherwise pass it along */
	          result.append(token);
		  pos += token.length();
		}
	    }
	}

      /* if we were still inside a tag, its not a tag */
      if (inside)
	{
	  result.append("<");
	  result.append(tag_word);
	  pos += 1 + tag_word.length();
	}

      /* end current effect region */
      if ((cur_color != null || cur_font != Font.PLAIN) && pos >= effect_start)
	{
	  result_eff.addElement(new text_effect_record(
			      effect_start,pos,cur_font, cur_color));
	}

      return result.toString();
    }
  
  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Potentially adjust first_vis_char and first_vis_effect based on new 
   * offset.  If this moves the first_vis_char index over, it will also 
   * move over the offset by the width of the characters removed.
   */
  protected void trim_front_chars()
    {
      text_effect_record eff;

      /* let the super class figure out the new first_vis_char */
      super.trim_front_chars();

      /* now skip over any effects we are now past */
      while (first_vis_effect < effect_list().size())
	{
	  /* extract the effect */
	  eff = (text_effect_record)effect_list().elementAt(first_vis_effect);

	  /* if it starts at or after the first vis char we keep it */
	  if (eff.start_pos >= first_vis_char) break;

	  /* if it ends after the first vis char we keep it */
	  if (eff.end_pos > first_vis_char) break;

	  /* otherwise we discard it */
	  first_vis_effect++;
	}
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Override to reset offset for text and effect when text is replaced. 
   * @param String txt tnew text to display.
   */
  public void set_text(String txt)
    {
      /* this gets called in super class initialization.  In that case, 
       * _effect_list hasn't been initialized properly, so we catch that 
       * here, before we try to parse tags into it. */
      _effect_list = new Vector();

      /* let super class set processed text */
      super.set_text(parse_tags(txt,_effect_list));

      /* reset our effect bookkeeping to put string back at the beginning */
      first_vis_effect = 0;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Draw the display 
   * @param drawable d the drawable on which to do the display
   */
  protected void draw_self_local(drawable d) 
{
      Rectangle clip_rect;
      int draw_index, draw_loc, draw_size;
      int clip_right, str_len;
      String str;
      int effect_index, rt_end;
      text_effect_record eff;
      boolean inside;

      Font plain       = font(); 
      Font bold        = restyle(font(), Font.BOLD); 
      Font italic      = restyle(font(), Font.ITALIC); 
      Font bold_italic = restyle(font(), Font.ITALIC | Font.BOLD);

      /* clear the background if they asked for that */
      if (opaque())
	{
          d.setColor(draw_colors().background());
          d.fillRect(0,0,w()-1,h()-1);
	}

      /* set font to our font, and start drawing in foreground */
      d.setFont(plain);
      d.setColor(draw_colors().foreground());

      /* draw the text in chunks starting at first_vis_char until we run off 
       * the end of the clipping area.  each chuck runs either to an effect
       * change, the end of the string, or 10 characters, which ever is less.
       */
      str_len = text().length();
      draw_index = first_vis_char; 
      effect_index = first_vis_effect;
      clip_rect = d.getClipRect();
      clip_right = clip_rect.x + clip_rect.width;
      for (draw_loc = h_spacing()-x_offset; 
	   draw_loc <= clip_right && draw_index < str_len; 
	   draw_loc += draw_size)
	{
	  /* do we have an effect to apply */
	  if (effect_index < effect_list().size())
	    {
	      /* determine if start is inside the first effect, or before it */
	      eff = (text_effect_record)effect_list().elementAt(effect_index);
	      inside = draw_index >= eff.start_pos;

	      /* work out index just past right end */
	      rt_end = Math.min(draw_index+10,str_len);
	      if (inside)
	        rt_end = Math.min(rt_end, eff.end_pos);
	      else
	        rt_end = Math.min(rt_end, eff.start_pos);
	    }
	  else
	    {
	      eff = null;
	      inside = false;

	      /* used fixed chunk of 10 characters */
	      rt_end = Math.min(draw_index+10,str_len);
	    }

	  /* start out with default drawing state */
          d.setColor(draw_colors().foreground());
          d.setFont(plain); 

	  /* establish drawing state for effect if we are inside it */
	  if (inside)
	    {
	      if (eff.effect_color != null)
		d.setColor(eff.effect_color);

	      switch(eff.effect_font)
		{
		  default:                    d.setFont(plain); break;
		  case Font.BOLD:             d.setFont(bold); break;
		  case Font.ITALIC:           d.setFont(italic); break;
		  case Font.ITALIC|Font.BOLD: d.setFont(bold_italic); break;
		}
	    }

	  /* pull out the chunk of string and draw it */
          str = text().substring(draw_index, rt_end);
          d.drawString(str, draw_loc, v_spacing()+_metric.getAscent());

	  /* set up for the next chunk */
	  draw_index = rt_end;
          draw_size = _metric.stringWidth(str);
	  if (eff != null && draw_index >= eff.end_pos) effect_index++;
	}

      /* draw box if requested */
      d.setColor(Color.black);
      if (boxed())
        d.drawRect(0,0,w()-1,h()-1);
    }
}
   //had:
   //* @exception general PROPAGATED

/*-----------------------------------------------------------------------*/

/** 
 * "Local" class for recording text effects to be applied to sliding text. 
 * This records the staring and ending effect, a font style for the effect 
 * and a color for the effect.
 *
 * @author Scott Hudson
 */
class text_effect_record {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Starting position (inclusive) of effect */
  public int start_pos = 0;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Ending position (exclusive) of effect */
  public int end_pos = 0;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Font style for this effect */
  public int effect_font = Font.PLAIN;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Color for effect, or null if this is to use normal color */
  public Color effect_color = null;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Full constructor.
   * @param int   start_p start position of the effect.
   * @param int   end_p   end position of the effect.
   * @param int   fnt     indicator for the font drawing style to use.
   * @param Color clr     the color for the effect.
   */
  public text_effect_record(int start_p, int end_p, int fnt, Color clr)
    {
      start_pos    = start_p;
      end_pos      = end_p;
      effect_font  = fnt;
      effect_color = clr;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
